@@ -29,9 +29,9 @@ enum {
   l_throttle_last,
 };
 
-Throttle::Throttle(CephContext *cct, std::string n, int64_t m, bool _use_perf)
+Throttle::Throttle(CephContext *cct, const std::string& n, int64_t m, bool _use_perf)
   : cct(cct), name(n), logger(NULL),
-		max(m),
+    max(m),
     lock("Throttle::lock"),
     use_perf(_use_perf)
 {
@@ -40,22 +40,24 @@ Throttle::Throttle(CephContext *cct, std::string n, int64_t m, bool _use_perf)
   if (!use_perf)
     return;
 
-  PerfCountersBuilder b(cct, string("throttle-") + name, l_throttle_first, l_throttle_last);
-  b.add_u64_counter(l_throttle_val, "val");
-  b.add_u64_counter(l_throttle_max, "max");
-  b.add_u64_counter(l_throttle_get, "get");
-  b.add_u64_counter(l_throttle_get_sum, "get_sum");
-  b.add_u64_counter(l_throttle_get_or_fail_fail, "get_or_fail_fail");
-  b.add_u64_counter(l_throttle_get_or_fail_success, "get_or_fail_success");
-  b.add_u64_counter(l_throttle_take, "take");
-  b.add_u64_counter(l_throttle_take_sum, "take_sum");
-  b.add_u64_counter(l_throttle_put, "put");
-  b.add_u64_counter(l_throttle_put_sum, "put_sum");
-  b.add_time_avg(l_throttle_wait, "wait");
+  if (cct->_conf->throttler_perf_counter) {
+    PerfCountersBuilder b(cct, string("throttle-") + name, l_throttle_first, l_throttle_last);
+    b.add_u64_counter(l_throttle_val, "val", "Currently available throttle");
+    b.add_u64_counter(l_throttle_max, "max", "Max value for throttle");
+    b.add_u64_counter(l_throttle_get, "get", "Gets");
+    b.add_u64_counter(l_throttle_get_sum, "get_sum", "Got data");
+    b.add_u64_counter(l_throttle_get_or_fail_fail, "get_or_fail_fail", "Get blocked during get_or_fail");
+    b.add_u64_counter(l_throttle_get_or_fail_success, "get_or_fail_success", "Successful get during get_or_fail");
+    b.add_u64_counter(l_throttle_take, "take", "Takes");
+    b.add_u64_counter(l_throttle_take_sum, "take_sum", "Taken data");
+    b.add_u64_counter(l_throttle_put, "put", "Puts");
+    b.add_u64_counter(l_throttle_put_sum, "put_sum", "Put data");
+    b.add_time_avg(l_throttle_wait, "wait", "Waiting latency");
 
-  logger = b.create_perf_counters();
-  cct->get_perfcounters_collection()->add(logger);
-  logger->set(l_throttle_max, max.read());
+    logger = b.create_perf_counters();
+    cct->get_perfcounters_collection()->add(logger);
+    logger->set(l_throttle_max, max.read());
+  }
 }
 
 Throttle::~Throttle()
@@ -69,13 +71,17 @@ Throttle::~Throttle()
   if (!use_perf)
     return;
 
-  cct->get_perfcounters_collection()->remove(logger);
-  delete logger;
+  if (logger) {
+    cct->get_perfcounters_collection()->remove(logger);
+    delete logger;
+  }
 }
 
 void Throttle::_reset_max(int64_t m)
 {
   assert(lock.is_locked());
+  if ((int64_t)max.read() == m)
+    return;
   if (!cond.empty())
     cond.front()->SignalOne();
   if (logger)
@@ -93,7 +99,8 @@ bool Throttle::_wait(int64_t c)
     do {
       if (!waited) {
 	ldout(cct, 2) << "_wait waiting..." << dendl;
-	start = ceph_clock_now(cct);
+	if (logger)
+	  start = ceph_clock_now(cct);
       }
       waited = true;
       cv->Wait(lock);
@@ -101,9 +108,10 @@ bool Throttle::_wait(int64_t c)
 
     if (waited) {
       ldout(cct, 3) << "_wait finished waiting" << dendl;
-      utime_t dur = ceph_clock_now(cct) - start;
-      if (logger)
+      if (logger) {
+	utime_t dur = ceph_clock_now(cct) - start;
         logger->tinc(l_throttle_wait, dur);
+      }
     }
 
     delete cv;
@@ -118,6 +126,10 @@ bool Throttle::_wait(int64_t c)
 
 bool Throttle::wait(int64_t m)
 {
+  if (0 == max.read() && 0 == m) {
+    return false;
+  }
+
   Mutex::Locker l(lock);
   if (m) {
     assert(m > 0);
@@ -129,6 +141,9 @@ bool Throttle::wait(int64_t m)
 
 int64_t Throttle::take(int64_t c)
 {
+  if (0 == max.read()) {
+    return 0;
+  }
   assert(c >= 0);
   ldout(cct, 10) << "take " << c << dendl;
   {
@@ -145,6 +160,10 @@ int64_t Throttle::take(int64_t c)
 
 bool Throttle::get(int64_t c, int64_t m)
 {
+  if (0 == max.read() && 0 == m) {
+    return false;
+  }
+
   assert(c >= 0);
   ldout(cct, 10) << "get " << c << " (" << count.read() << " -> " << (count.read() + c) << ")" << dendl;
   bool waited = false;
@@ -170,6 +189,10 @@ bool Throttle::get(int64_t c, int64_t m)
  */
 bool Throttle::get_or_fail(int64_t c)
 {
+  if (0 == max.read()) {
+    return true;
+  }
+
   assert (c >= 0);
   Mutex::Locker l(lock);
   if (_should_wait(c) || !cond.empty()) {
@@ -193,6 +216,10 @@ bool Throttle::get_or_fail(int64_t c)
 
 int64_t Throttle::put(int64_t c)
 {
+  if (0 == max.read()) {
+    return 0;
+  }
+
   assert(c >= 0);
   ldout(cct, 10) << "put " << c << " (" << count.read() << " -> " << (count.read()-c) << ")" << dendl;
   Mutex::Locker l(lock);
@@ -242,6 +269,12 @@ void SimpleThrottle::end_op(int r)
   m_cond.Signal();
 }
 
+bool SimpleThrottle::pending_error() const
+{
+  Mutex::Locker l(m_lock);
+  return (m_ret < 0);
+}
+
 int SimpleThrottle::wait_for_ret()
 {
   Mutex::Locker l(m_lock);
@@ -249,3 +282,89 @@ int SimpleThrottle::wait_for_ret()
     m_cond.Wait(m_lock);
   return m_ret;
 }
+
+void C_OrderedThrottle::finish(int r) {
+  m_ordered_throttle->finish_op(m_tid, r);
+}
+
+OrderedThrottle::OrderedThrottle(uint64_t max, bool ignore_enoent)
+  : m_lock("OrderedThrottle::m_lock"), m_max(max), m_current(0), m_ret_val(0),
+    m_ignore_enoent(ignore_enoent), m_next_tid(0), m_complete_tid(0) {
+}
+
+C_OrderedThrottle *OrderedThrottle::start_op(Context *on_finish) {
+  assert(on_finish != NULL);
+
+  Mutex::Locker locker(m_lock);
+  uint64_t tid = m_next_tid++;
+  m_tid_result[tid] = Result(on_finish);
+  C_OrderedThrottle *ctx = new C_OrderedThrottle(this, tid);
+
+  complete_pending_ops();
+  while (m_max == m_current) {
+    m_cond.Wait(m_lock);
+    complete_pending_ops();
+  }
+  ++m_current;
+
+  return ctx;
+}
+
+void OrderedThrottle::end_op(int r) {
+  Mutex::Locker locker(m_lock);
+  assert(m_current > 0);
+
+  if (r < 0 && m_ret_val == 0 && (r != -ENOENT || !m_ignore_enoent)) {
+    m_ret_val = r;
+  }
+  --m_current;
+  m_cond.Signal();
+}
+
+void OrderedThrottle::finish_op(uint64_t tid, int r) {
+  Mutex::Locker locker(m_lock);
+
+  TidResult::iterator it = m_tid_result.find(tid);
+  assert(it != m_tid_result.end());
+
+  it->second.finished = true;
+  it->second.ret_val = r;
+  m_cond.Signal();
+}
+
+bool OrderedThrottle::pending_error() const {
+  Mutex::Locker locker(m_lock);
+  return (m_ret_val < 0);
+}
+
+int OrderedThrottle::wait_for_ret() {
+  Mutex::Locker locker(m_lock);
+  complete_pending_ops();
+
+  while (m_current > 0) {
+    m_cond.Wait(m_lock);
+    complete_pending_ops();
+  }
+  return m_ret_val;
+}
+
+void OrderedThrottle::complete_pending_ops() {
+  assert(m_lock.is_locked());
+
+  while (true) {
+    TidResult::iterator it = m_tid_result.begin();
+    if (it == m_tid_result.end() || it->first != m_complete_tid ||
+        !it->second.finished) {
+      break;
+    }
+
+    Result result = it->second;
+    m_tid_result.erase(it);
+
+    m_lock.Unlock();
+    result.on_finish->complete(result.ret_val);
+    m_lock.Lock();
+
+    ++m_complete_tid;
+  }
+}
