@@ -13,7 +13,7 @@
  */
 
 #include "SnapServer.h"
-#include "MDS.h"
+#include "MDSRank.h"
 #include "osd/OSDMap.h"
 #include "osdc/Objecter.h"
 #include "mon/MonClient.h"
@@ -29,7 +29,7 @@
 
 #define dout_subsys ceph_subsys_mds
 #undef dout_prefix
-#define dout_prefix *_dout << "mds." << mds->get_nodeid() << ".snap "
+#define dout_prefix *_dout << "mds." << rank << ".snap "
 
 
 void SnapServer::reset_state()
@@ -37,6 +37,28 @@ void SnapServer::reset_state()
   last_snap = 1;  /* snapid 1 reserved for initial root snaprealm */
   snaps.clear();
   need_to_purge.clear();
+
+  // find any removed snapshot in data pools
+  if (mds) {  // only if I'm running in a live MDS
+    snapid_t first_free = 0;
+    const OSDMap *osdmap = mds->objecter->get_osdmap_read();
+    for (set<int64_t>::const_iterator p = mds->mdsmap->get_data_pools().begin();
+         p != mds->mdsmap->get_data_pools().end();
+         ++p) {
+      const pg_pool_t *pi = osdmap->get_pg_pool(*p);
+      if (!pi) {
+        // If pool isn't in OSDMap yet then can't have any snaps needing
+        // removal, skip.
+        continue;
+      }
+      if (!pi->removed_snaps.empty() &&
+          pi->removed_snaps.range_end() > first_free)
+        first_free = pi->removed_snaps.range_end();
+    }
+    mds->objecter->put_osdmap_read();
+    if (first_free > last_snap)
+      last_snap = first_free;
+  }
 }
 
 
@@ -59,7 +81,8 @@ void SnapServer::_prepare(bufferlist &bl, uint64_t reqid, mds_rank_t bymds)
 	::decode(info.name, p);
 	::decode(info.stamp, p);
 	info.snapid = ++last_snap;
-	pending_create[version] = info;
+	info.long_name = "create";
+	pending_update[version] = info;
 	dout(10) << "prepare v" << version << " create " << info << dendl;
       } else {
 	pending_noop.insert(version);
@@ -89,6 +112,26 @@ void SnapServer::_prepare(bufferlist &bl, uint64_t reqid, mds_rank_t bymds)
     }
     break;
 
+  case TABLE_OP_UPDATE:
+    {
+      SnapInfo info;
+      ::decode(info.ino, p);
+      ::decode(info.snapid, p);
+      ::decode(info.name, p);
+      ::decode(info.stamp, p);
+      info.long_name = "update";
+
+      version++;
+      // bump last_snap... we use it as a version value on the snaprealm.
+      ++last_snap;
+      pending_update[version] = info;
+      dout(10) << "prepare v" << version << " update " << info << dendl;
+
+      bl.clear();
+      ::encode(last_snap, bl);
+    }
+    break;
+
   default:
     assert(0);
   }
@@ -98,17 +141,25 @@ void SnapServer::_prepare(bufferlist &bl, uint64_t reqid, mds_rank_t bymds)
 bool SnapServer::_is_prepared(version_t tid)
 {
   return 
-    pending_create.count(tid) ||
+    pending_update.count(tid) ||
     pending_destroy.count(tid);
 }
 
 bool SnapServer::_commit(version_t tid, MMDSTableRequest *req)
 {
-  if (pending_create.count(tid)) {
-    dout(7) << "commit " << tid << " create " << pending_create[tid] << dendl;
-    snaps[pending_create[tid].snapid] = pending_create[tid];
-    pending_create.erase(tid);
-  } 
+  if (pending_update.count(tid)) {
+    SnapInfo &info = pending_update[tid];
+    string opname;
+    if (info.long_name.empty())
+      opname = "create";
+    else
+      opname.swap(info.long_name);
+    if (info.stamp == utime_t() && snaps.count(info.snapid))
+      info.stamp = snaps[info.snapid].stamp;
+    dout(7) << "commit " << tid << " " << opname << " " << info << dendl;
+    snaps[info.snapid] = info;
+    pending_update.erase(tid);
+  }
 
   else if (pending_destroy.count(tid)) {
     snapid_t sn = pending_destroy[tid].first;
@@ -140,9 +191,15 @@ bool SnapServer::_commit(version_t tid, MMDSTableRequest *req)
 
 void SnapServer::_rollback(version_t tid) 
 {
-  if (pending_create.count(tid)) {
-    dout(7) << "rollback " << tid << " create " << pending_create[tid] << dendl;
-    pending_create.erase(tid);
+  if (pending_update.count(tid)) {
+    SnapInfo &info = pending_update[tid];
+    string opname;
+    if (info.long_name.empty())
+      opname = "create";
+    else
+      opname.swap(info.long_name);
+    dout(7) << "rollback " << tid << " " << opname << " " << info << dendl;
+    pending_update.erase(tid);
   } 
 
   else if (pending_destroy.count(tid)) {
@@ -208,6 +265,12 @@ void SnapServer::check_osd_map(bool force)
        ++p) {
     int id = p->first;
     const pg_pool_t *pi = osdmap->get_pg_pool(id);
+    if (pi == NULL) {
+      // The pool is gone.  So are the snapshots.
+      all_purged[id] = std::vector<snapid_t>(p->second.begin(), p->second.end());
+      continue;
+    }
+
     for (set<snapid_t>::iterator q = p->second.begin();
 	 q != p->second.end();
 	 ++q) {
@@ -231,7 +294,7 @@ void SnapServer::check_osd_map(bool force)
   if (!all_purge.empty()) {
     dout(10) << "requesting removal of " << all_purge << dendl;
     MRemoveSnaps *m = new MRemoveSnaps(all_purge);
-    mds->monc->send_mon_message(m);
+    mon_client->send_mon_message(m);
   }
 
   last_checked_osdmap = version;
@@ -270,8 +333,8 @@ void SnapServer::dump(Formatter *f) const
   }
   f->close_section();
 
-  f->open_array_section("pending_create");
-  for(map<version_t, SnapInfo>::const_iterator i = pending_create.begin(); i != pending_create.end(); ++i) {
+  f->open_array_section("pending_update");
+  for(map<version_t, SnapInfo>::const_iterator i = pending_update.begin(); i != pending_update.end(); ++i) {
     f->open_object_section("snap");
     f->dump_unsigned("version", i->first);
     f->open_object_section("snapinfo");
@@ -309,7 +372,7 @@ void SnapServer::generate_test_instances(list<SnapServer*>& ls)
   populated->last_snap = 123;
   populated->snaps[456] = populated_snapinfo;
   populated->need_to_purge[2].insert(012);
-  populated->pending_create[234] = populated_snapinfo;
+  populated->pending_update[234] = populated_snapinfo;
   populated->pending_destroy[345].first = 567;
   populated->pending_destroy[345].second = 768;
   populated->pending_noop.insert(890);
