@@ -17,7 +17,7 @@
 
 #include "osdc/Objecter.h"
 #include "osdc/Filer.h"
-#include "mds/MDS.h"
+#include "mds/MDSRank.h"
 #include "mds/MDCache.h"
 #include "mds/MDLog.h"
 #include "mds/CDir.h"
@@ -30,14 +30,14 @@
 #define dout_subsys ceph_subsys_mds
 #undef dout_prefix
 #define dout_prefix _prefix(_dout, mds)
-static ostream& _prefix(std::ostream *_dout, MDS *mds) {
+static ostream& _prefix(std::ostream *_dout, MDSRank *mds) {
   return *_dout << "mds." << mds->get_nodeid() << ".cache.strays ";
 }
 
 class StrayManagerIOContext : public virtual MDSIOContextBase {
 protected:
   StrayManager *sm;
-  virtual MDS *get_mds()
+  virtual MDSRank *get_mds()
   {
     return sm->mds;
   }
@@ -49,7 +49,7 @@ public:
 class StrayManagerContext : public virtual MDSInternalContextBase {
 protected:
   StrayManager *sm;
-  virtual MDS *get_mds()
+  virtual MDSRank *get_mds()
   {
     return sm->mds;
   }
@@ -94,7 +94,7 @@ void StrayManager::purge(CDentry *dn, uint32_t op_allowance)
   C_GatherBuilder gather(
     g_ceph_context,
     new C_OnFinisher(new C_IO_PurgeStrayPurged(
-        this, dn, false, op_allowance), &mds->finisher));
+        this, dn, false, op_allowance), mds->finisher));
 
   if (in->is_dir()) {
     object_locator_t oloc(mds->mdsmap->get_metadata_pool());
@@ -551,7 +551,7 @@ bool StrayManager::__eval_stray(CDentry *dn, bool delay)
     // past snaprealm parents imply snapped dentry remote links.
     // only important for directories.  normal file data snaps are handled
     // by the object store.
-    if (in->snaprealm && in->snaprealm->has_past_parents()) {
+    if (in->snaprealm) {
       if (!in->snaprealm->have_past_parents_open() &&
           !in->snaprealm->open_parents(new C_MDC_EvalStray(this, dn))) {
         return false;
@@ -747,15 +747,15 @@ void StrayManager::migrate_stray(CDentry *dn, mds_rank_t to)
   mds->send_message_mds(req, to);
 }
 
-StrayManager::StrayManager(MDS *mds)
+StrayManager::StrayManager(MDSRank *mds)
   : delayed_eval_stray(member_offset(CDentry, item_stray)),
     mds(mds), logger(NULL),
     ops_in_flight(0), files_purging(0),
+    max_purge_ops(0), 
     num_strays(0), num_strays_purging(0), num_strays_delayed(0),
-    filer(mds->objecter, &mds->finisher)
+    filer(mds->objecter, mds->finisher)
 {
   assert(mds != NULL);
-  update_op_limit();
 }
 
 void StrayManager::abort_queue()
@@ -794,7 +794,7 @@ void StrayManager::truncate(CDentry *dn, uint32_t op_allowance)
   C_GatherBuilder gather(
     g_ceph_context,
     new C_OnFinisher(new C_IO_PurgeStrayPurged(this, dn, true, 0),
-		     &mds->finisher));
+		     mds->finisher));
 
   SnapRealm *realm = in->find_snaprealm();
   assert(realm);
@@ -867,12 +867,21 @@ void StrayManager::handle_conf_change(const struct md_config_t *conf,
 void StrayManager::update_op_limit()
 {
   const OSDMap *osdmap = mds->objecter->get_osdmap_read();
+  assert(osdmap != NULL);
 
   // Number of PGs across all data pools
   uint64_t pg_count = 0;
   const std::set<int64_t> &data_pools = mds->mdsmap->get_data_pools();
   for (std::set<int64_t>::iterator i = data_pools.begin();
        i != data_pools.end(); ++i) {
+    if (osdmap->get_pg_pool(*i) == NULL) {
+      // It is possible that we have an older OSDMap than MDSMap, because
+      // we don't start watching every OSDMap until after MDSRank is
+      // initialized
+      dout(4) << __func__ << " data pool " << *i
+              << " not found in OSDMap" << dendl;
+      continue;
+    }
     pg_count += osdmap->get_pg_num(*i);
   }
 
