@@ -178,11 +178,6 @@ public:
     int *exist
     );
 
-  /// @see CollectionIndex
-  int collection_list(
-    vector<ghobject_t> *ls
-    );
-
   /// @see CollectionIndex;
   int pre_hash_collection(
       uint32_t pg_num,
@@ -192,9 +187,9 @@ public:
   /// @see CollectionIndex
   int collection_list_partial(
     const ghobject_t &start,
-    int min_count,
+    const ghobject_t &end,
+    bool sort_bitwise,
     int max_count,
-    snapid_t seq,
     vector<ghobject_t> *ls,
     ghobject_t *next
     );
@@ -243,21 +238,6 @@ protected:
     int *exists		  ///< [out] True if the object exists.
     ) = 0;
 
-  /**
-   * List contents of the collection, must be implemented by derived class.
-   *
-   * @param [out] seq Snapid to list.
-   * @param [in] max_count Max number to list (0 for no limit).
-   * @param [out] ls Container for listed objects.
-   * @param [in,out] last List handle.  0 for beginning.  Passing the same
-   * cookie location will cause the next max_count to be listed.
-   * @return Error code.  0 on success.
-   */
-  /// List contents of collection.
-  virtual int _collection_list(
-    vector<ghobject_t> *ls ///< [out] Listed objects.
-    ) = 0;
-
   /// Pre-hash the collection with the given pg number and
   /// expected number of objects in the collection.
   virtual int _pre_hash_collection(
@@ -268,9 +248,9 @@ protected:
   /// @see CollectionIndex
   virtual int _collection_list_partial(
     const ghobject_t &start,
-    int min_count,
+    const ghobject_t &end,
+    bool sort_bitwise,
     int max_count,
-    snapid_t seq,
     vector<ghobject_t> *ls,
     ghobject_t *next
     ) = 0;
@@ -385,7 +365,7 @@ protected:
   /// Lists subdirectories.
   int list_subdirs(
     const vector<string> &to_list, ///< [in] Directory to list.
-    set<string> *out		   ///< [out] Subdirectories listed. 
+    vector<string> *out		   ///< [out] Subdirectories listed.
     );
 
   /// Create subdirectory.
@@ -576,12 +556,6 @@ private:
     const string &attr ///< [in] Attribute to mangle.
     ); ///< @return Mangled attribute name.
 
-  /// checks whether long_name could hash to short_name
-  bool short_name_matches(
-    const char *short_name,    ///< [in] name to check against
-    const char *cand_long_name ///< [in] candidate long name
-    );
-
   /// Builds hashed filename
   void build_filename(
     const char *old_filename, ///< [in] Filename to convert.
