@@ -1070,9 +1070,12 @@ uint64_t OSDMap::get_features(int entity_type, uint64_t *pmask) const
 	 ++p) {
       const map<string,string> &profile = p->second;
       map<string,string>::const_iterator plugin = profile.find("plugin");
-      if (plugin != profile.end() && (plugin->second == "isa" ||
-				      plugin->second == "lrc"))
-	features |= CEPH_FEATURE_ERASURE_CODE_PLUGINS_V2;
+      if (plugin != profile.end()) {
+	if (plugin->second == "isa" || plugin->second == "lrc")
+	  features |= CEPH_FEATURE_ERASURE_CODE_PLUGINS_V2;
+	if (plugin->second == "shec")
+	  features |= CEPH_FEATURE_ERASURE_CODE_PLUGINS_V3;
+      }
     }
   }
   mask |= CEPH_FEATURE_OSDHASHPSPOOL | CEPH_FEATURE_OSD_CACHEPOOL;
@@ -2227,15 +2230,6 @@ void OSDMap::dump_erasure_code_profiles(const map<string,map<string,string> > &p
   f->close_section();
 }
 
-void OSDMap::dump_json(ostream& out) const
-{
-  JSONFormatter jsf(true);
-  jsf.open_object_section("osdmap");
-  dump(&jsf);
-  jsf.close_section();
-  jsf.flush(out);
-}
-
 void OSDMap::dump(Formatter *f) const
 {
   f->dump_int("epoch", get_epoch());
@@ -2322,7 +2316,7 @@ void OSDMap::dump(Formatter *f) const
   }
   f->close_section(); // primary_temp
 
-  f->open_array_section("blacklist");
+  f->open_object_section("blacklist");
   for (ceph::unordered_map<entity_addr_t,utime_t>::const_iterator p = blacklist.begin();
        p != blacklist.end();
        ++p) {
@@ -2381,6 +2375,8 @@ string OSDMap::get_flag_string(unsigned f)
     s += ",nodeep-scrub";
   if (f & CEPH_OSDMAP_NOTIERAGENT)
     s += ",notieragent";
+  if (f & CEPH_OSDMAP_SORTBITWISE)
+    s += ",sortbitwise";
   if (s.length())
     s.erase(0, 1);
   return s;
@@ -2565,15 +2561,16 @@ private:
   const OSDMap *osdmap;
 };
 
-void OSDMap::print_tree(ostream *out, Formatter *f) const
+void OSDMap::print_tree(Formatter *f, ostream *out) const
 {
-  if (out) {
+  if (f)
+    OSDTreeFormattingDumper(crush.get(), this).dump(f);
+  else {
+    assert(out);
     TextTable tbl;
     OSDTreePlainDumper(crush.get(), this).dump(&tbl);
     *out << tbl;
   }
-  if (f)
-    OSDTreeFormattingDumper(crush.get(), this).dump(f);
 }
 
 void OSDMap::print_summary(Formatter *f, ostream& out) const
@@ -2675,6 +2672,7 @@ int OSDMap::build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
     r = build_simple_crush_map(cct, *crush, nosd, &ss);
   else
     r = build_simple_crush_map_from_conf(cct, *crush, &ss);
+  assert(r == 0);
 
   int poolbase = get_max_osd() ? get_max_osd() : 1;
 
@@ -2705,9 +2703,6 @@ int OSDMap::build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
     name_pool[*p] = pool;
   }
 
-  if (r < 0)
-    lderr(cct) << ss.str() << dendl;
-  
   for (int i=0; i<get_max_osd(); i++) {
     set_state(i, 0);
     set_weight(i, CEPH_OSD_OUT);
@@ -2730,8 +2725,6 @@ int OSDMap::get_erasure_code_profile_default(CephContext *cct,
   int r = get_json_str_map(cct->_conf->osd_pool_default_erasure_code_profile,
 		      *ss,
 		      &profile_map);
-  profile_map["directory"] =
-    cct->_conf->osd_pool_default_erasure_code_directory;
   return r;
 }
 
@@ -2861,180 +2854,21 @@ int OSDMap::build_simple_crush_rulesets(CephContext *cct,
 					const string& root,
 					ostream *ss)
 {
+  int crush_ruleset =
+      crush._get_osd_pool_default_crush_replicated_ruleset(cct, true);
   string failure_domain =
     crush.get_type_name(cct->_conf->osd_crush_chooseleaf_type);
 
+  if (crush_ruleset == CEPH_DEFAULT_CRUSH_REPLICATED_RULESET)
+    crush_ruleset = -1; // create ruleset 0 by default
+
   int r;
-  r = crush.add_simple_ruleset("replicated_ruleset", root, failure_domain,
-			       "firstn", pg_pool_t::TYPE_REPLICATED, ss);
+  r = crush.add_simple_ruleset_at("replicated_ruleset", root, failure_domain,
+                                  "firstn", pg_pool_t::TYPE_REPLICATED,
+                                  crush_ruleset, ss);
   if (r < 0)
     return r;
   // do not add an erasure rule by default or else we will implicitly
   // require the crush_v2 feature of clients
   return 0;
 }
-
-int OSDMap::summarize_mapping_stats(
-  OSDMap *newmap,
-  const set<int64_t> *pools,
-  std::string *out,
-  Formatter *f) const
-{
-  set<int64_t> ls;
-  if (pools) {
-    ls = *pools;
-  } else {
-    for (map<int64_t, pg_pool_t>::const_iterator i = get_pools().begin();
-	 i != get_pools().end();
-	 ++i) {
-      ls.insert(i->first);
-    }
-  }
-
-  unsigned total_pg = 0;
-  unsigned moved_pg = 0;
-  vector<unsigned> base_by_osd(get_max_osd(), 0);
-  vector<unsigned> new_by_osd(get_max_osd(), 0);
-  for (set<int64_t>::iterator p = ls.begin(); p != ls.end(); ++p) {
-    int64_t pool_id = *p;
-    const pg_pool_t *pi = get_pg_pool(pool_id);
-    vector<int> up, up2, acting;
-    int up_primary, acting_primary;
-    for (unsigned ps = 0; ps < pi->get_pg_num(); ++ps) {
-      pg_t pgid(ps, pool_id, -1);
-      total_pg += pi->get_size();
-      pg_to_up_acting_osds(pgid, &up, &up_primary,
-			   &acting, &acting_primary);
-      for (vector<int>::iterator q = up.begin(); q != up.end(); ++q) {
-	int osd = *q;
-	if (osd >= 0 && osd < get_max_osd())
-	  ++base_by_osd[osd];
-      }
-      if (newmap) {
-	newmap->pg_to_up_acting_osds(pgid, &up2, &up_primary,
-				     &acting, &acting_primary);
-	for (vector<int>::iterator q = up2.begin(); q != up2.end(); ++q) {
-	  int osd = *q;
-	  if (osd >= 0 && osd < get_max_osd())
-	    ++new_by_osd[osd];
-	}
-	if (pi->type == pg_pool_t::TYPE_ERASURE) {
-	  for (unsigned i=0; i<up.size(); ++i) {
-	    if (up[i] != up2[i]) {
-	      ++moved_pg;
-	    }
-	  }
-	} else if (pi->type == pg_pool_t::TYPE_REPLICATED) {
-	  for (vector<int>::iterator q = up.begin(); q != up.end(); ++q) {
-	    int osd = *q;
-	    if (std::find(up2.begin(), up2.end(), osd) == up2.end()) {
-	      ++moved_pg;
-	    }
-	  }
-	} else {
-	  assert(0 == "unhandled pool type");
-	}
-      }
-    }
-  }
-
-  unsigned num_up_in = 0;
-  for (int osd = 0; osd < get_max_osd(); ++osd) {
-    if (is_up(osd) && is_in(osd))
-      ++num_up_in;
-  }
-  if (!num_up_in) {
-    return -EINVAL;
-  }
-
-  float avg_pg = (float)total_pg / (float)num_up_in;
-  float base_stddev = 0, new_stddev = 0;
-  int min = -1, max = -1;
-  unsigned min_base_pg = 0, max_base_pg = 0;
-  unsigned min_new_pg = 0, max_new_pg = 0;
-  for (int osd = 0; osd < get_max_osd(); ++osd) {
-    if (is_up(osd) && is_in(osd)) {
-      float base_diff = (float)base_by_osd[osd] - avg_pg;
-      base_stddev += base_diff * base_diff;
-      float new_diff = (float)new_by_osd[osd] - avg_pg;
-      new_stddev += new_diff * new_diff;
-      if (min < 0 || min_base_pg < base_by_osd[osd]) {
-	min = osd;
-	min_base_pg = base_by_osd[osd];
-	min_new_pg = new_by_osd[osd];
-      }
-      if (max < 0 || max_base_pg > base_by_osd[osd]) {
-	max = osd;
-	max_base_pg = base_by_osd[osd];
-	max_new_pg = new_by_osd[osd];
-      }
-    }
-  }
-  base_stddev = sqrt(base_stddev / num_up_in);
-  new_stddev = sqrt(new_stddev / num_up_in);
-
-  float edev = sqrt(avg_pg * (1.0 - (1.0 / (double)num_up_in)));
-
-  ostringstream ss;
-  if (f)
-    f->open_object_section("utilization");
-  if (newmap) {
-    if (f) {
-      f->dump_unsigned("moved_pgs", moved_pg);
-      f->dump_unsigned("total_pgs", total_pg);
-    } else {
-      ss << "moved " << moved_pg << " / " << total_pg
-	 << " (" << ((float)moved_pg * 100.0 / (float)total_pg) << "%)\n";
-    }
-  }
-  if (f) {
-    f->dump_float("avg_pgs", avg_pg);
-    f->dump_float("std_dev", base_stddev);
-    f->dump_float("expected_baseline_std_dev", edev);
-    if (newmap)
-      f->dump_float("new_std_dev", new_stddev);
-  } else {
-    ss << "avg " << avg_pg << "\n";
-    ss << "stddev " << base_stddev;
-    if (newmap)
-      ss << " -> " << new_stddev;
-    ss << " (expected baseline " << edev << ")\n";
-  }
-  if (min >= 0) {
-    if (f) {
-      f->dump_unsigned("min_osd", min);
-      f->dump_unsigned("min_osd_pgs", min_base_pg);
-      if (newmap)
-	f->dump_unsigned("new_min_osd_pgs", min_new_pg);
-    } else {
-      ss << "min osd." << min << " with " << min_base_pg;
-      if (newmap)
-	ss << " -> " << min_new_pg;
-      ss << " pgs (" << (float)min_base_pg / avg_pg;
-      if (newmap)
-	ss << " -> " << (float)min_new_pg / avg_pg;
-      ss << " * mean)\n";
-    }
-  }
-  if (max >= 0) {
-    if (f) {
-      f->dump_unsigned("max_osd", max);
-      f->dump_unsigned("max_osd_pgs", max_base_pg);
-      if (newmap)
-	f->dump_unsigned("new_max_osd_pgs", max_new_pg);
-    } else {
-      ss << "max osd." << max << " with " << max_base_pg;
-      if (newmap)
-	ss << " -> " << max_new_pg;
-      ss << " pgs (" << (float)max_base_pg / avg_pg;
-      if (newmap)
-	ss << " -> " << (float)max_new_pg / avg_pg;
-      ss << " * mean)\n";
-    }
-  }
-  if (f)
-    f->close_section();
-  if (out)
-    *out = ss.str();
-  return 0;
-}
