@@ -13,6 +13,7 @@
 
 #ifdef NDEBUG
 #define TEST_SYNC_POINT(x)
+#define TEST_SYNC_POINT_CALLBACK(x, y)
 #else
 
 namespace rocksdb {
@@ -39,7 +40,8 @@ class SyncPoint {
   void LoadDependency(const std::vector<Dependency>& dependencies);
 
   // Set up a call back function in sync point.
-  void SetCallBack(const std::string point, std::function<void()> callback);
+  void SetCallBack(const std::string point,
+                   std::function<void(void*)> callback);
   // Clear all call back functions.
   void ClearAllCallBacks();
 
@@ -54,7 +56,8 @@ class SyncPoint {
 
   // triggered by TEST_SYNC_POINT, blocking execution until all predecessors
   // are executed.
-  void Process(const std::string& point);
+  // And/or call registered callback functionn, with argument `cb_arg`
+  void Process(const std::string& point, void* cb_arg = nullptr);
 
   // TODO: it might be useful to provide a function that blocks until all
   // sync points are cleared.
@@ -65,7 +68,7 @@ class SyncPoint {
   // successor/predecessor map loaded from LoadDependency
   std::unordered_map<std::string, std::vector<std::string>> successors_;
   std::unordered_map<std::string, std::vector<std::string>> predecessors_;
-  std::unordered_map<std::string, std::function<void()> > callbacks_;
+  std::unordered_map<std::string, std::function<void(void*)> > callbacks_;
 
   std::mutex mutex_;
   std::condition_variable cv_;
@@ -84,4 +87,6 @@ class SyncPoint {
 // See TransactionLogIteratorRace in db_test.cc for an example use case.
 // TEST_SYNC_POINT is no op in release build.
 #define TEST_SYNC_POINT(x) rocksdb::SyncPoint::GetInstance()->Process(x)
+#define TEST_SYNC_POINT_CALLBACK(x, y) \
+  rocksdb::SyncPoint::GetInstance()->Process(x, y)
 #endif  // NDEBUG
